@@ -4,7 +4,7 @@
 
     Test templating.
 
-    :copyright: Copyright 2007-2019 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2020 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -20,7 +20,7 @@ def test_layout_overloading(make_app, app_params):
     setup_documenters(app)
     app.builder.build_update()
 
-    result = (app.outdir / 'index.html').text()
+    result = (app.outdir / 'index.html').read_text()
     assert '<!-- layout overloading -->' in result
 
 
@@ -31,5 +31,5 @@ def test_autosummary_class_template_overloading(make_app, app_params):
     setup_documenters(app)
     app.builder.build_update()
 
-    result = (app.outdir / 'generated' / 'sphinx.application.TemplateBridge.html').text()
+    result = (app.outdir / 'generated' / 'sphinx.application.TemplateBridge.html').read_text()
     assert 'autosummary/class.rst method block overloading' in result
