@@ -13,6 +13,7 @@ import datetime
 import functools
 import sys
 import types
+from inspect import Parameter
 
 import pytest
 
@@ -20,76 +21,6 @@ from sphinx.util import inspect
 from sphinx.util.inspect import stringify_signature
 
 
-def test_getargspec():
-    def func(a, b, c=1, d=2, *e, **f):
-        pass
-
-    spec = inspect.getargspec(func)
-    assert spec.args == ['a', 'b', 'c', 'd']
-    assert spec.varargs == 'e'
-    assert spec.varkw == 'f'
-    assert spec.defaults == (1, 2)
-    assert spec.kwonlyargs == []
-    assert spec.kwonlydefaults is None
-    assert spec.annotations == {}
-
-
-def test_getargspec_partial():
-    def func1(a, b, c=1, d=2, *e, **f):
-        pass
-
-    partial = functools.partial(func1, 10, c=11)
-    spec = inspect.getargspec(partial)
-    assert spec.args == ['b']
-    assert spec.varargs is None
-    assert spec.varkw == 'f'
-    assert spec.defaults is None
-    assert spec.kwonlyargs == ['c', 'd']
-    assert spec.kwonlydefaults == {'c': 11, 'd': 2}
-    assert spec.annotations == {}
-
-
-def test_getargspec_partial2():
-    def fun(a, b, c=1, d=2):
-        pass
-    p = functools.partial(fun, 10, c=11)
-
-    def f_expected(b, *, c=11, d=2):
-        pass
-    expected = inspect.getargspec(f_expected)
-
-    assert expected == inspect.getargspec(p)
-
-
-def test_getargspec_builtin_type():
-    with pytest.raises(TypeError):
-        inspect.getargspec(int)
-
-
-def test_getargspec_bound_methods():
-    def f_expected_unbound(self, arg1, **kwargs):
-        pass
-    expected_unbound = inspect.getargspec(f_expected_unbound)
-
-    def f_expected_bound(arg1, **kwargs):
-        pass
-    expected_bound = inspect.getargspec(f_expected_bound)
-
-    class Foo:
-        def method(self, arg1, **kwargs):
-            pass
-
-    bound_method = Foo().method
-
-    @functools.wraps(bound_method)
-    def wrapped_bound_method(*args, **kwargs):
-        pass
-
-    assert expected_unbound == inspect.getargspec(Foo.method)
-    assert expected_bound == inspect.getargspec(bound_method)
-    assert expected_bound == inspect.getargspec(wrapped_bound_method)
-
-
 def test_signature():
     # literals
     with pytest.raises(TypeError):
@@ -298,17 +229,108 @@ def test_signature_annotations():
 @pytest.mark.skipif(sys.version_info < (3, 8), reason='python 3.8+ is required.')
 @pytest.mark.sphinx(testroot='ext-autodoc')
 def test_signature_annotations_py38(app):
-    from target.pep570 import foo, bar
+    from target.pep570 import foo, bar, baz, qux
+
+    # case: separator at head
+    sig = inspect.signature(foo)
+    assert stringify_signature(sig) == '(*, a, b)'
 
     # case: separator in the middle
-    sig = inspect.signature(foo)
+    sig = inspect.signature(bar)
     assert stringify_signature(sig) == '(a, b, /, c, d)'
 
+    sig = inspect.signature(baz)
+    assert stringify_signature(sig) == '(a, /, *, b)'
+
     # case: separator at tail
-    sig = inspect.signature(bar)
+    sig = inspect.signature(qux)
     assert stringify_signature(sig) == '(a, b, /)'
 
 
+def test_signature_from_str_basic():
+    signature = '(a, b, *args, c=0, d="blah", **kwargs)'
+    sig = inspect.signature_from_str(signature)
+    assert list(sig.parameters.keys()) == ['a', 'b', 'args', 'c', 'd', 'kwargs']
+    assert sig.parameters['a'].name == 'a'
+    assert sig.parameters['a'].kind == Parameter.POSITIONAL_OR_KEYWORD
+    assert sig.parameters['a'].default == Parameter.empty
+    assert sig.parameters['a'].annotation == Parameter.empty
+    assert sig.parameters['b'].name == 'b'
+    assert sig.parameters['b'].kind == Parameter.POSITIONAL_OR_KEYWORD
+    assert sig.parameters['b'].default == Parameter.empty
+    assert sig.parameters['b'].annotation == Parameter.empty
+    assert sig.parameters['args'].name == 'args'
+    assert sig.parameters['args'].kind == Parameter.VAR_POSITIONAL
+    assert sig.parameters['args'].default == Parameter.empty
+    assert sig.parameters['args'].annotation == Parameter.empty
+    assert sig.parameters['c'].name == 'c'
+    assert sig.parameters['c'].kind == Parameter.KEYWORD_ONLY
+    assert sig.parameters['c'].default == '0'
+    assert sig.parameters['c'].annotation == Parameter.empty
+    assert sig.parameters['d'].name == 'd'
+    assert sig.parameters['d'].kind == Parameter.KEYWORD_ONLY
+    assert sig.parameters['d'].default == "'blah'"
+    assert sig.parameters['d'].annotation == Parameter.empty
+    assert sig.parameters['kwargs'].name == 'kwargs'
+    assert sig.parameters['kwargs'].kind == Parameter.VAR_KEYWORD
+    assert sig.parameters['kwargs'].default == Parameter.empty
+    assert sig.parameters['kwargs'].annotation == Parameter.empty
+    assert sig.return_annotation == Parameter.empty
+
+
+def test_signature_from_str_default_values():
+    signature = ('(a=0, b=0.0, c="str", d=b"bytes", e=..., f=True, '
+                 'g=[1, 2, 3], h={"a": 1}, i={1, 2, 3}, '
+                 'j=lambda x, y: None, k=None, l=object(), m=foo.bar.CONSTANT)')
+    sig = inspect.signature_from_str(signature)
+    assert sig.parameters['a'].default == '0'
+    assert sig.parameters['b'].default == '0.0'
+    assert sig.parameters['c'].default == "'str'"
+    assert sig.parameters['d'].default == "b'bytes'"
+    assert sig.parameters['e'].default == '...'
+    assert sig.parameters['f'].default == 'True'
+    assert sig.parameters['g'].default == '[1, 2, 3]'
+    assert sig.parameters['h'].default == "{'a': 1}"
+    assert sig.parameters['i'].default == '{1, 2, 3}'
+    assert sig.parameters['j'].default == 'lambda x, y: ...'
+    assert sig.parameters['k'].default == 'None'
+    assert sig.parameters['l'].default == 'object()'
+    assert sig.parameters['m'].default == 'foo.bar.CONSTANT'
+
+
+def test_signature_from_str_annotations():
+    signature = '(a: int, *args: bytes, b: str = "blah", **kwargs: float) -> None'
+    sig = inspect.signature_from_str(signature)
+    assert list(sig.parameters.keys()) == ['a', 'args', 'b', 'kwargs']
+    assert sig.parameters['a'].annotation == "int"
+    assert sig.parameters['args'].annotation == "bytes"
+    assert sig.parameters['b'].annotation == "str"
+    assert sig.parameters['kwargs'].annotation == "float"
+    assert sig.return_annotation == 'None'
+
+
+def test_signature_from_str_complex_annotations():
+    sig = inspect.signature_from_str('() -> Tuple[str, int, ...]')
+    assert sig.return_annotation == 'Tuple[str, int, ...]'
+
+    sig = inspect.signature_from_str('() -> Callable[[int, int], int]')
+    assert sig.return_annotation == 'Callable[[int, int], int]'
+
+
+@pytest.mark.skipif(sys.version_info < (3, 8),
+                    reason='python-3.8 or above is required')
+def test_signature_from_str_positionaly_only_args():
+    sig = inspect.signature_from_str('(a, /, b)')
+    assert list(sig.parameters.keys()) == ['a', 'b']
+    assert sig.parameters['a'].kind == Parameter.POSITIONAL_ONLY
+    assert sig.parameters['b'].kind == Parameter.POSITIONAL_OR_KEYWORD
+
+
+def test_signature_from_str_invalid():
+    with pytest.raises(SyntaxError):
+        inspect.signature_from_str('')
+
+
 def test_safe_getattr_with_default():
     class Foo:
         def __getattr__(self, item):
@@ -412,6 +434,16 @@ def test_dict_customtype():
     assert "<CustomType(2)>: 2" in description
 
 
+@pytest.mark.sphinx(testroot='ext-autodoc')
+def test_isclassmethod(app):
+    from target.methods import Base, Inherited
+
+    assert inspect.isclassmethod(Base.classmeth) is True
+    assert inspect.isclassmethod(Base.meth) is False
+    assert inspect.isclassmethod(Inherited.classmeth) is True
+    assert inspect.isclassmethod(Inherited.meth) is False
+
+
 @pytest.mark.sphinx(testroot='ext-autodoc')
 def test_isstaticmethod(app):
     from target.methods import Base, Inherited
